@@ -1,5 +1,5 @@
 use crate::language::Language;
-use crate::match_tree::{extract_var_from_node, match_end_non_recursive, match_node_non_recursive};
+use crate::match_tree::{match_end_non_recursive, match_node_non_recursive, MatchStrictness};
 use crate::matcher::{KindMatcher, KindMatcherError, Matcher};
 use crate::meta_var::{MetaVarEnv, MetaVariable};
 use crate::source::TSParseError;
@@ -13,10 +13,17 @@ use std::collections::HashSet;
 use std::marker::PhantomData;
 
 #[derive(Clone)]
-pub enum Pattern<L: Language> {
+pub struct Pattern<L: Language> {
+  pub node: PatternNode,
+  root_kind: Option<u16>,
+  lang: PhantomData<L>,
+  pub strictness: MatchStrictness,
+}
+
+#[derive(Clone)]
+pub enum PatternNode {
   MetaVar {
     meta_var: MetaVariable,
-    kind: Option<u16>,
   },
   /// Node without children.
   Terminal {
@@ -27,35 +34,77 @@ pub enum Pattern<L: Language> {
   /// Non-Terminal Syntax Nodes are called Internal
   Internal {
     kind_id: u16,
-    children: Vec<Pattern<L>>,
-    lang: PhantomData<L>,
+    children: Vec<PatternNode>,
   },
 }
 
+impl PatternNode {
+  // for skipping trivial nodes in goal after ellipsis
+  pub fn is_trivial(&self) -> bool {
+    match self {
+      PatternNode::Terminal { is_named, .. } => !*is_named,
+      _ => false,
+    }
+  }
+
+  pub fn fixed_string(&self) -> Cow<str> {
+    match &self {
+      PatternNode::Terminal { text, .. } => Cow::Borrowed(text),
+      PatternNode::MetaVar { .. } => Cow::Borrowed(""),
+      PatternNode::Internal { children, .. } => {
+        children
+          .iter()
+          .map(|n| n.fixed_string())
+          .fold(Cow::Borrowed(""), |longest, curr| {
+            if longest.len() >= curr.len() {
+              longest
+            } else {
+              curr
+            }
+          })
+      }
+    }
+  }
+}
+impl<'r, D: Doc> From<Node<'r, D>> for PatternNode {
+  fn from(node: Node<'r, D>) -> Self {
+    convert_node_to_pattern(node)
+  }
+}
+
 impl<'r, D: Doc> From<Node<'r, D>> for Pattern<D::Lang> {
   fn from(node: Node<'r, D>) -> Self {
-    convert_node_to_pattern(node, None)
+    Self {
+      node: convert_node_to_pattern(node),
+      root_kind: None,
+      lang: PhantomData,
+      strictness: MatchStrictness::Smart,
+    }
   }
 }
 
-fn convert_node_to_pattern<D: Doc>(node: Node<D>, kind: Option<u16>) -> Pattern<D::Lang> {
+fn convert_node_to_pattern<D: Doc>(node: Node<D>) -> PatternNode {
   if let Some(meta_var) = extract_var_from_node(&node) {
-    Pattern::MetaVar { meta_var, kind }
+    PatternNode::MetaVar { meta_var }
   } else if node.is_leaf() {
-    Pattern::Terminal {
+    PatternNode::Terminal {
       text: node.text().to_string(),
       is_named: node.is_named(),
       kind_id: node.kind_id(),
     }
   } else {
-    Pattern::Internal {
+    PatternNode::Internal {
       kind_id: node.kind_id(),
-      children: node.children().map(Pattern::from).collect(),
-      lang: PhantomData,
+      children: node.children().map(PatternNode::from).collect(),
     }
   }
 }
 
+fn extract_var_from_node<D: Doc>(goal: &Node<D>) -> Option<MetaVariable> {
+  let key = goal.text();
+  goal.lang().extract_meta_var(&key)
+}
+
 #[derive(Debug, Error)]
 pub enum PatternError {
   #[error("Tree-Sitter fails to parse the pattern.")]
@@ -88,51 +137,27 @@ impl<L: Language> Pattern<L> {
     Self::new(src, lang)
   }
 
-  pub fn fixed_string(&self) -> Cow<str> {
-    match self {
-      Self::Terminal { text, .. } => Cow::Borrowed(text),
-      Self::MetaVar { .. } => Cow::Borrowed(""),
-      Self::Internal { children, .. } => {
-        children
-          .iter()
-          .map(|n| n.fixed_string())
-          .fold(Cow::Borrowed(""), |longest, curr| {
-            if longest.len() >= curr.len() {
-              longest
-            } else {
-              curr
-            }
-          })
-      }
-    }
-  }
-
   pub fn has_error(&self) -> bool {
-    let kind = match self {
-      Pattern::Terminal { kind_id, .. } => *kind_id,
-      Pattern::Internal { kind_id, .. } => *kind_id,
-      Pattern::MetaVar {
-        kind: Some(kind_id),
-        ..
-      } => *kind_id,
-      Pattern::MetaVar { kind: None, .. } => return false,
+    let kind = match &self.node {
+      PatternNode::Terminal { kind_id, .. } => *kind_id,
+      PatternNode::Internal { kind_id, .. } => *kind_id,
+      PatternNode::MetaVar { .. } => match self.root_kind {
+        Some(k) => k,
+        None => return false,
+      },
     };
     KindMatcher::<L>::from_id(kind).is_error_matcher()
   }
 
-  // for skipping trivial nodes in goal after ellipsis
-  pub fn is_trivial(&self) -> bool {
-    match self {
-      Pattern::Terminal { is_named, .. } => !*is_named,
-      _ => false,
-    }
+  pub fn fixed_string(&self) -> Cow<str> {
+    self.node.fixed_string()
   }
 
   /// Get all defined variables in the pattern.
   /// Used for validating rules and report undefined variables.
   pub fn defined_vars(&self) -> HashSet<&str> {
     let mut vars = HashSet::new();
-    collect_vars(self, &mut vars);
+    collect_vars(&self.node, &mut vars);
     vars
   }
 }
@@ -147,17 +172,17 @@ fn meta_var_name(meta_var: &MetaVariable) -> Option<&str> {
   }
 }
 
-fn collect_vars<'p, L: Language>(p: &'p Pattern<L>, vars: &mut HashSet<&'p str>) {
+fn collect_vars<'p>(p: &'p PatternNode, vars: &mut HashSet<&'p str>) {
   match p {
-    Pattern::MetaVar { meta_var, .. } => {
+    PatternNode::MetaVar { meta_var, .. } => {
       if let Some(name) = meta_var_name(meta_var) {
         vars.insert(name);
       }
     }
-    Pattern::Terminal { .. } => {
+    PatternNode::Terminal { .. } => {
       // collect nothing for terminal nodes!
     }
-    Pattern::Internal { children, .. } => {
+    PatternNode::Internal { children, .. } => {
       for c in children {
         collect_vars(c, vars);
       }
@@ -195,10 +220,12 @@ impl<L: Language> Pattern<L> {
         selector: selector.into(),
       });
     };
-    Ok(convert_node_to_pattern(
-      node.get_node().clone(),
-      Some(node.kind_id()),
-    ))
+    Ok(Self {
+      root_kind: Some(node.kind_id()),
+      node: convert_node_to_pattern(node.get_node().clone()),
+      lang: PhantomData,
+      strictness: MatchStrictness::Smart,
+    })
   }
   pub fn doc(doc: StrDoc<L>) -> Self {
     let root = Root::doc(doc);
@@ -232,11 +259,12 @@ impl<L: Language> Matcher<L> for Pattern<L> {
   }
 
   fn potential_kinds(&self) -> Option<bit_set::BitSet> {
-    let kind = match self {
-      Self::Terminal { kind_id, .. } => *kind_id,
-      Self::MetaVar { kind, .. } => (*kind)?,
-      Self::Internal { kind_id, .. } => *kind_id,
+    let kind = match self.node {
+      PatternNode::Terminal { kind_id, .. } => kind_id,
+      PatternNode::MetaVar { .. } => self.root_kind?,
+      PatternNode::Internal { kind_id, .. } => kind_id,
     };
+
     let mut kinds = BitSet::new();
     kinds.insert(kind.into());
     Some(kinds)
@@ -248,8 +276,7 @@ impl<L: Language> Matcher<L> for Pattern<L> {
     Some(end - start)
   }
 }
-
-impl<L: Language> std::fmt::Debug for Pattern<L> {
+impl std::fmt::Debug for PatternNode {
   fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
     match self {
       Self::MetaVar { meta_var, .. } => write!(f, "{:?}", meta_var),
@@ -259,6 +286,12 @@ impl<L: Language> std::fmt::Debug for Pattern<L> {
   }
 }
 
+impl<L: Language> std::fmt::Debug for Pattern<L> {
+  fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    write!(f, "{:?}", self.node)
+  }
+}
+
 #[cfg(test)]
 mod test {
   use super::*;
