@@ -180,7 +180,7 @@ impl SgNode {
         reference.inner.find(pattern)
       }
       Either3::C(config) => {
-        let pattern = config.parse_with(lang)?;
+        let pattern = config.parse_with(lang.into())?;
         reference.inner.find(pattern)
       }
     };
@@ -220,7 +220,7 @@ impl SgNode {
         reference.inner.find_all(pattern).collect()
       }
       Either3::C(config) => {
-        let pattern = config.parse_with(lang)?;
+        let pattern = config.parse_with(lang.into())?;
         reference.inner.find_all(pattern).collect()
       }
     };
