@@ -7,11 +7,17 @@ class Pattern(TypedDict):
     selector: str
     context: str
 
+class NthChild(TypedDict):
+    position: int | str
+    ofRule: Rule
+    nth: int
+
 class RuleWithoutNot(TypedDict, total=False):
     # atomic rule
     pattern: str | Pattern
     kind: str
     regex: str
+    nthChild: int | str | NthChild
 
     # relational rule
     inside: "Relation" # pyright report error if forward reference here?
@@ -51,6 +57,7 @@ __all__ = [
     "Config",
     "Relation",
     "Pattern",
+    "NthChild",
     "SgNode",
     "SgRoot",
     "Pos",
