@@ -3,7 +3,7 @@
 mod py_node;
 mod range;
 mod unicode_position;
-use py_node::SgNode;
+use py_node::{Edit, SgNode};
 use range::{Pos, Range};
 
 use ast_grep_core::{AstGrep, Language, NodeMatch, StrDoc};
@@ -19,6 +19,7 @@ fn ast_grep_py(_py: Python, m: &Bound<PyModule>) -> PyResult<()> {
   m.add_class::<SgNode>()?;
   m.add_class::<Range>()?;
   m.add_class::<Pos>()?;
+  m.add_class::<Edit>()?;
   Ok(())
 }
 
